#include <GL/glut.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define screen_width 640
#define screen_height 480

#define PI 3.1456

#define nb_metaballs 20

unsigned char back_buffer[ screen_width*screen_height];
unsigned char texture_map[257][257];


typedef struct{
    float r,g,b;
}color_type;

color_type palette[256];


void Init_Palette()
{
	int i;
	/* pas mal, plus moleculaire
	for( i = 0 ; i< 70 ; i++)
	{
		palette[i].r=i/70.0;
		palette[i].g=i/70.0;
		palette[i].b=0.0;
	}
	for( i = 70 ; i< 256 ; i++)
	{
		palette[i].r=0.9+((i-70)*0.1)/186.0;
		palette[i].g=0.9+((i-70)*0.1)/186.0;
		palette[i].b= (i-70)/186.0;
	}*/

	/*
	// violet
	for( i = 0 ; i< 200 ; i++)
	{
		palette[i].r=i/200.0;
		palette[i].b=i/200.0;
		palette[i].g=0.0;
	}
	for( i = 200 ; i< 256 ; i++)
	{
		palette[i].r=1.0;
		palette[i].b=1.0;
		palette[i].g=1.0*(i-200)/56.0;
	}*/
	for( i = 0 ; i< 200 ; i++)
	{
		palette[i].r=i/200.0;
		palette[i].b=0.0;
		palette[i].g=0.0;
	}
	for( i = 200 ; i< 256 ; i++)
	{
		palette[i].r=1.0;
		palette[i].b=1.0*(i-200)/56.0;
		palette[i].g=1.0*(i-200)/56.0;
	}
}

class Metaballs2D
{
public:
	Metaballs2D(int px , int py , int rayon)
		{
			unsigned char t=screen_width/320;
			Px=px;
			Py=py;
			Rayon=rayon;
			do{
				Vx=rand()%(t*20)-t*10;
				Vy=rand()%(t*20)-t*10;
			}while( Vx==0 && Vy==0);
			printf("new metaballs rayon=%d \n",Rayon);
		}

	void AddToBackground()
	{
		unsigned char r =  (unsigned char) Rayon;
		unsigned char rapport=7;

		while( !(r & 0x01 ) )
		{
			r=r>>1;
			rapport--;
		}

		unsigned char potential;
		//printf("add meta2D rayon=%d rapport=%d \n" , Rayon , rapport);
		for(int x=0 ; x<2*Rayon+1 ; x++)
			for(int y=0 ; y<2*Rayon+1 ; y++)
				if( ! ( Px-Rayon+x <0 || Px-Rayon+x>= screen_width|| Py-Rayon+y <0 || Py-Rayon+y>=screen_height) )
				{
					potential = texture_map[ x << rapport][y<<rapport];
					if( back_buffer[ Px-Rayon+x + (Py - Rayon +y ) *screen_width] + potential <= 255)
						back_buffer[ Px-Rayon+x + (Py - Rayon +y ) *screen_width] += potential;
					else
						back_buffer[ Px-Rayon+x + (Py - Rayon +y ) *screen_width] =255;
				}
	}

	void Move()
	{
		if( (Px + Vx) < 0 || (Px+Vx) >=screen_width || (Py + Vy) < 0 || (Py+Vy) >=screen_height)
		{
			unsigned char t=screen_width/320;
			do{
				Vx=rand()%(t*20)-t*10;
				Vy=rand()%(t*20)-t*10;
			}while( Vx==0 && Vy==0);

		}
		else
		{
			Px += Vx;
			Py += Vy;
		}
	}
private:
	int Px,Py,Rayon;
	int Vx,Vy;
};



Metaballs2D * tab_meta[20];



void Init_Texture()
{
  printf("texture a initialiser ! \n");
  double l;
  float potential;
  for(int x=-128 ; x <= 128 ; x++)
     for(int y=-128 ; y <= 128 ; y++)
       if( x*x+y*y < 128*128)
	     {
	     l = sqrt( x*x + y*y);
	     //potential = (unsigned char)( cos_table[ (int) ((l/128)*360.0) ] *255  );
	     potential = ( 160+cos( (l/128)*PI)*160  );
	     if( potential > 255.0) potential = 255.0;
	     texture_map[x+128][y+128] = (unsigned char) potential;
	     }
	else
		texture_map[x+128][y+128] = 0;

  printf("texture initialise ! \n");
}


void display(void)
{
  glClear(GL_COLOR_BUFFER_BIT);
  memset( &back_buffer ,(unsigned char) 0 ,  screen_width *  screen_height);

  for(int i=0;i<nb_metaballs;i++)
  	tab_meta[i]->Move();
  for(int i=0;i<nb_metaballs;i++)
  	tab_meta[i]->AddToBackground();

  glBegin(GL_POINTS);
  for(int x=0;x<screen_width;x++)
    for(int y=0;y<screen_height;y++)
      if( back_buffer[x+y*screen_width] != 0 )
      {
	glColor3f( palette[back_buffer[x+y*screen_width]].r ,
		   palette[back_buffer[x+y*screen_width]].g ,
		   palette[back_buffer[x+y*screen_width]].b );

	glVertex2i( x , y);
      }
  glEnd();
  glutSwapBuffers(); // In double buffered mode we invert the positions of the visible buffer and the writing buffer

}


void resize(int w, int h)
{
  glViewport( 0 , 0 ,(GLsizei)w , (GLsizei)h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D( 0.0 , (GLdouble) w , 0.0 , (GLdouble)h);
}


void init(void)
{
  glClearColor(0.0,0.0,0.0,0.0);
  memset( &back_buffer ,(unsigned char) 0 ,  screen_width *  screen_height);
  Init_Texture();
  Init_Palette();
  srand ( time(NULL) );
  for(int i=0;i<nb_metaballs;i++)
  	tab_meta[i] = new Metaballs2D( rand()%(screen_width/3) + screen_width/2 , rand()%(screen_height/3) + screen_height/2 , 1<<(rand()%2+4+screen_width/320) );
}


int main(int argc, char **argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(screen_width,screen_height);
    glutInitWindowPosition(100,100);
    glutCreateWindow("Metaballs 2D");
    glutDisplayFunc(display);
    glutReshapeFunc (resize);
    glutIdleFunc(display);
    init();
    glutMainLoop();

    return(0);
}





